

CREATE OR REPLACE FUNCTION spRevaluasi(
	_P_RevaDate	DATE
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	cData		REFCURSOR;
	cSubData	REFCURSOR;

	_CrcId		crc.crcid%TYPE;
	_PerId		per.perid%TYPE;

	_CurExcRate	NUMERIC(19,4);
	_ExcRate	NUMERIC(19,4);
	_SelisihRate	NUMERIC(19,4);

	_CashId		cash.cashid%TYPE;
	_CashName	cash.cashdesc%TYPE;
	_AccCash	acc.accno%TYPE;
	_CashBal	NUMERIC(19,4);
	_BaseCashBal	NUMERIC(19,4);
	_SelisihCashBal	NUMERIC(19,4);

	_RefNo		VARCHAR(25);
	_BPId		bp.bpid%TYPE;
	_Sisa		NUMERIC(19,4);

	_JnlBalikNo	jnl.jnlno%TYPE;
	_JnlBalikDate	DATE;
	_JnlBalikNote	TEXT;
	_JnlNo		jnl.jnlno%TYPE;
	_JnlNote	TEXT;
BEGIN
	_retval:=(0, 'OK');

	OPEN cData FOR
		SELECT crcid, excrate 
		FROM crc 
		WHERE isdefault=FALSE;
	LOOP
		FETCH cData INTO _CrcId, _CurExcRate;
		EXIT WHEN NOT FOUND;

		/** Proses Kas Valas */
		OPEN cSubData FOR
			SELECT c.cashid, c.cashdesc, c.accno, c.cashbal*_CurExcRate, c.basecashbal
			FROM cash c 
			WHERE c.crcid=_CrcId AND c.cashbal!=0 OR (c.cashbal=0 AND c.basecashbal!=0);
		LOOP
			FETCH cSubData INTO  _CashId, _CashName, _AccCash, _CashBal, _BaseCashBal;
			EXIT WHEN NOT FOUND;

			_SelisihCashBal:=_CashBal-_BaseCashBal;

			SELECT INTO _spret * FROM spCashReva_New(_P_RevaDate, _CashId, _CashName, _AccCash, _CurExcRate, _CurExcRate, _PerId, _CrcId, _BaseCashBal, _SelisihCashBal);
			IF (_spret.ErrNo!=0) THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;
		CLOSE cSubData;

		/** Proses Purc Kredit aka Hutang Valas */
		OPEN cSubData FOR
			SELECT purcno, total-paidamt, vendorid, excrate 
			FROM purc 
			WHERE crcid=_CrcId AND paystatus<>'F' 
			ORDER BY purcno;
		LOOP
			FETCH cSubData INTO _RefNo, _Sisa, _BPId, _ExcRate;
			EXIT WHEN NOT FOUND;

			SELECT INTO _JnlBalikNo, _JnlBalikDate jnlno, jnldate 
			FROM jnl 
			WHERE jnltype='REVA' AND reftype='PURC' AND refno=_RefNo AND jnldate<_P_RevaDate 
			AND UPPER(jnlnote) LIKE 'REVALUASI KURS%' AND (_xt IS NULL OR _xt=FALSE) 
			ORDER BY jnldate DESC 
			LIMIT 1;
			IF FOUND THEN
				_JnlBalikNote:='Jurnal Pembalik atas revaluasi kurs pembelian#'||_RefNo||E'\nper tanggal '||_JnlBalikDate||' (ref#'||_JnlBalikNo||')';
				SELECT INTO _spret * FROM spJurnalPembalik_Revaluasi(_JnlBalikNo, _JnlBalikNote, _P_RevaDate, 'PURC', _RefNo);
				IF (_spret.ErrNo<>0) THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF; 

			_SelisihRate:=_CurExcRate-_ExcRate;
			_JnlNo:=fTransNoGet('JURNAL');
			_JnlNote:= 'Revaluasi Kurs Pembelian'||':'||_RefNo;
			SELECT INTO _spret * FROM spJnl_Insert(_JnlNo, 'REVA', _P_RevaDate, 'PURC', _RefNo,
				_JnlNote, TRUE, TRUE, CAST(0 AS NUMERIC(19,4)), NULL, NULL, NULL, NULL,
				_PerId, NULL, NULL, NULL, NULL, _CrcId, _CurExcRate, _CurExcRate
			);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF(_SelisihRate>0) THEN
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, 1, fAccDefault('ACCPROFIT'), 'D', _SelisihRate*_Sisa, NULL, NULL, _SelisihRate*_Sisa);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo,2, fBpAcc(_BPId, _CrcId, 'AP'), 'K', _SelisihRate*_Sisa, NULL, NULL, _SelisihRate*_Sisa);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			ELSIF(_SelisihRate<0) THEN
				_SelisihRate:=ABS(_SelisihRate);

				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo,1, fBpAcc(_BPId, _CrcId, 'AP'), 'D', _SelisihRate*_Sisa, NULL, NULL, _SelisihRate*_Sisa);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, 2, fAccDefault('ACCPROFIT'), 'K', _SelisihRate*_Sisa, NULL, NULL, _SelisihRate*_Sisa);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;

			SELECT INTO _spret * FROM spJnl_New(_JnlNo);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;
		CLOSE cSubData;

		/** Proses Sale Kredit aka Piutang Valas */
		OPEN cSubData FOR
			SELECT saleno, total-paidamt, custid, excrate 
			FROM sale 
			WHERE crcid=_CrcId AND paystatus<>'F' 
			ORDER BY saleno;
		LOOP
			FETCH cSubData INTO _RefNo, _Sisa, _BPId, _ExcRate;
			EXIT WHEN NOT FOUND;

			SELECT INTO _JnlBalikNo, _JnlBalikDate jnlno, jnldate 
			FROM jnl 
			WHERE jnltype='REVA' AND reftype='SALE' AND refno=_RefNo AND jnldate<_P_RevaDate 
			AND UPPER(jnlnote) LIKE 'REVALUASI KURS%' AND (_xt IS NULL OR _xt=FALSE) 
			ORDER BY jnldate DESC 
			LIMIT 1;
			IF FOUND THEN
				_JnlBalikNote:='Jurnal Pembalik atas revaluasi kurs penjualan#'||_RefNo||E'\nper tanggal '||_JnlBalikDate||' (ref#'||_JnlBalikNo||')';
				SELECT INTO _spret * FROM spJurnalPembalik_Revaluasi(_JnlBalikNo, _JnlBalikNote, _P_RevaDate, 'SALE', _RefNo);
				IF (_spret.ErrNo<>0) THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF; 

			_SelisihRate:=_CurExcRate-_ExcRate;
			_JnlNo:=fTransNoGet('JURNAL');
			_JnlNote:= 'Revaluasi Kurs Penjualan'||':'||_RefNo;
			SELECT INTO _spret * FROM spJnl_Insert(_JnlNo, 'REVA', _P_RevaDate, 'SALE', _RefNo,
				_JnlNote, TRUE, TRUE, CAST(0 AS NUMERIC(19,4)), NULL, NULL, NULL, NULL,
				_PerId, NULL, NULL, NULL,NULL, _CrcId, _CurExcRate, _CurExcRate
			);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF(_SelisihRate>0) THEN
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, 1, fAccDefault('ACCPROFIT'), 'D', _SelisihRate*_Sisa, NULL, NULL, _SelisihRate*_Sisa);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, 2, fBpAcc(_BPId, _CrcId, 'AR'), 'K', _SelisihRate*_Sisa, NULL, NULL, _SelisihRate*_Sisa);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			ELSIF(_SelisihRate<0) THEN
				_SelisihRate:=ABS(_SelisihRate);

				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, 1, fAccDefault('ACCPROFIT'), 'D', _SelisihRate*_Sisa, NULL, NULL, _SelisihRate*_Sisa);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, 2, fBpAcc(_BPId, _CrcId, 'AR'), 'K', _SelisihRate*_Sisa, NULL, NULL, _SelisihRate*_Sisa);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;

			SELECT INTO _spret * FROM spJnl_New(_JnlNo);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;
		CLOSE cSubData;

		/** Proses Retur Purc Kredit aka Retur Hutang Valas */
		OPEN cSubData FOR
			SELECT pretno, total, vendorid, excrate 
			FROM pret 
			WHERE crcid=_CrcId AND iscontra=false AND termtype='R' 
			ORDER BY pretno;
		LOOP
			FETCH cSubData INTO _RefNo, _Sisa, _BPId, _ExcRate;
			EXIT WHEN NOT FOUND;

			SELECT INTO _JnlBalikNo, _JnlBalikDate jnlno, jnldate 
			FROM jnl 
			WHERE jnltype='REVA' AND reftype='PRET' AND refno=_RefNo AND jnldate<_P_RevaDate 
			AND UPPER(jnlnote) LIKE 'REVALUASI KURS%' AND (_xt IS NULL OR _xt=FALSE) 
			ORDER BY jnldate DESC 
			LIMIT 1;
			IF FOUND THEN
				_JnlBalikNote:='Jurnal Pembalik atas revaluasi kurs retur pembelian#'||_RefNo||E'\nper tanggal '||_JnlBalikDate||' (ref#'||_JnlBalikNo||')';
				SELECT INTO _spret * FROM spJurnalPembalik_Revaluasi(_JnlBalikNo, _JnlBalikNote, _P_RevaDate, 'PRET', _RefNo);
				IF (_spret.ErrNo<>0) THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF; 

			_SelisihRate:=_CurExcRate-_ExcRate;
			_JnlNo:=fTransNoGet('JURNAL');
			_JnlNote:= 'Revaluasi Kurs Retur Pembelian'||':'||_RefNo;
			SELECT INTO _spret * FROM spJnl_Insert(_JnlNo, 'REVA', _P_RevaDate, 'PRET', _RefNo,
				_JnlNote, TRUE, TRUE, CAST(0 AS NUMERIC(19,4)), NULL, NULL, NULL, NULL,
				_PerId, NULL, NULL, NULL, NULL, _CrcId, _CurExcRate, _CurExcRate
			);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF(_SelisihRate>0) THEN
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, 1, fBpAcc(_BPId, _CrcId, 'AP'), 'D', _SelisihRate*_Sisa, NULL, NULL, _SelisihRate*_Sisa);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo,2, fAccDefault('ACCPROFIT'), 'K', _SelisihRate*_Sisa, NULL, NULL, _SelisihRate*_Sisa);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			ELSIF(_SelisihRate<0) THEN
				_SelisihRate:=ABS(_SelisihRate);

				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo,1, fAccDefault('ACCPROFIT'), 'D', _SelisihRate*_Sisa, NULL, NULL, _SelisihRate*_Sisa);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, 2, fBpAcc(_BPId, _CrcId,'AP') , 'K', _SelisihRate*_Sisa, NULL, NULL, _SelisihRate*_Sisa);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;

			SELECT INTO _spret * FROM spJnl_New(_JnlNo);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;
		CLOSE cSubData;

		/** Proses Retur Sale Kredit aka Retur Piutang Valas */
		OPEN cSubData FOR
			SELECT sretno, total, custid, excrate 
			FROM sret 
			WHERE crcid=_CrcId AND iscontra=false AND termtype='R' 
			ORDER BY sretno;
		LOOP
			FETCH cSubData INTO _RefNo, _Sisa, _BPId, _ExcRate;
			EXIT WHEN NOT FOUND;

			SELECT INTO _JnlBalikNo, _JnlBalikDate jnlno, jnldate 
			FROM jnl 
			WHERE jnltype='REVA' AND reftype='SRET' AND refno=_RefNo AND jnldate<_P_RevaDate 
			AND UPPER(jnlnote) LIKE 'REVALUASI KURS%' AND (_xt IS NULL OR _xt=FALSE) 
			ORDER BY jnldate DESC 
			LIMIT 1;
			IF FOUND THEN
				_JnlBalikNote:='Jurnal Pembalik atas revaluasi kurs retur penjualan#'||_RefNo||E'\nper tanggal '||_JnlBalikDate||' (ref#'||_JnlBalikNo||')';
				SELECT INTO _spret * FROM spJurnalPembalik_Revaluasi(_JnlBalikNo, _JnlBalikNote, _P_RevaDate, 'SRET', _RefNo);
				IF (_spret.ErrNo<>0) THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF; 

			_SelisihRate:=_CurExcRate-_ExcRate;
			_JnlNo:=fTransNoGet('JURNAL');
			_JnlNote:= 'Revaluasi Kurs Retur Penjualan'||':'||_RefNo;
			SELECT INTO _spret * FROM spJnl_Insert(_JnlNo, 'REVA', _P_RevaDate, 'SRET', _RefNo,
				_JnlNote, TRUE, TRUE, CAST(0 AS NUMERIC(19,4)), NULL, NULL, NULL, NULL,
				_PerId, NULL, NULL, NULL, NULL, _CrcId, _CurExcRate, _CurExcRate
			);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF (_SelisihRate>0) THEN
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, 1, fBpAcc(_BPId, _CrcId,'AR'), 'D', _SelisihRate*_Sisa, NULL, NULL, _SelisihRate*_Sisa);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, 2, fAccDefault('ACCPROFIT'), 'K', _SelisihRate*_Sisa, NULL, NULL, _SelisihRate*_Sisa);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			ELSIF (_SelisihRate<0) THEN
				_SelisihRate:=ABS(_SelisihRate);

				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, 1, fBpAcc(_BPId, _CrcId, 'AR') , 'D', _SelisihRate*_Sisa, NULL, NULL, _SelisihRate*_Sisa);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, 2, fAccDefault('ACCPROFIT'), 'K', _SelisihRate*_Sisa, NULL, NULL, _SelisihRate*_Sisa);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;

			SELECT INTO _spret * FROM spJnl_New(_JnlNo);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;
		CLOSE cSubData;
	END LOOP;
	CLOSE cData;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


